PROC Direct.Access.Device.Types(CHAN OF SP fs, ts,
                                CHAN OF TO.SCSI tscsi,
                                CHAN OF FROM.SCSI fscsi)
  --{{{  Declarations
  BOOL Going:
  INT  reply:
  INT  Before, After, Time.Taken:
  --}}}  
  --{{{  Implementation
  SEQ
    --{{{  Initialise
    Going := TRUE
    Before := 0
    After := 0
    Time.Taken := 0
    --}}}  
    WHILE Going
      SEQ
        --{{{  Prompt User
        Display.Options(fs, ts,
                        Main.Title,
                        "Direct Access Device Types (Chapter 8)",
                        ["Exit           ",
                         "Define Bus ID  ",
                         "Request Sense  ",
                         "Read Capacity  ",
                         "Read           ",
                         "Write          ",
                         "Verify         ",
                         "Start/Stop Unit",
                         "Seek           ",
                         "Format         "],
                         Status.Message,
                         reply)
        --}}}  
        --{{{  Do User request
        CASE reply
          0
            Going := FALSE
          1
            --{{{  Prompt operator for initiator and target SCSI device ID's
            BOOL First.Time:
            SEQ
              First.Time := TRUE
              Initiator.ID := 7 (BYTE)
              Target.ID := 7 (BYTE)
              WHILE (Initiator.ID = Target.ID)
                SEQ
                  IF
                    NOT First.Time
                      --{{{  Warning Message
                      so.write.string.nl(fs, ts, "Initiator ID and Target ID must be different.")
                      --}}}  
                    TRUE
                      First.Time := FALSE
                  --{{{  Prompt Operator for Initiator ID
                  INT reply:
                  SEQ
                    Quit := TRUE
                    so.ask(fs, ts,
                           "Which Initiator ID do you wish to set IMS B422 as > ",
                           "01234567",
                           TRUE,
                           TRUE,
                           reply)
                    Initiator.ID := BYTE reply
                    so.write.nl(fs, ts)
                    so.write.nl(fs, ts)
                  --}}}  
                  --{{{  Prompt Operator for Target ID
                  INT reply:
                  SEQ
                    Quit := TRUE
                    so.ask(fs, ts,
                           "Which Target ID do you wish IMS B422 to talk to   > ",
                           "01234567",
                           TRUE,
                           TRUE,
                           reply)
                    Target.ID := BYTE reply
                    so.write.nl(fs, ts)
                    so.write.nl(fs, ts)
                  --}}}  
              --{{{  Make IMS B422 an Initiator
              so.write.string.nl(fs, ts, "Make B422 an Initiator")
              Enable.Initiator.Mode( tscsi,
                                     fscsi,
                                     BYTE Initiator.ID,
                                     Execution.Status)
              --}}}  
              Prepare.Status.Message(Execution.Status, Status.Message)
            --}}}  
          2
            --{{{  Request Sense
            SEQ
              --{{{  Request Sense
              Request.Sense(fs, ts, tscsi, fscsi)
              --}}}  
            --}}}  
          3
            --{{{  Read Capacity
            SEQ
              Display.Line(fs, ts, "Reading Capacity of Drive")
              --{{{  Request Capacity from drive
              Set.Identify.Message()
              SCSI.Read.Capacity.10( tscsi,
                                     fscsi,
                                     Target.ID,
                                     LUN,
                                     Zero.B,
                                     0 (INT32),
                                     Zero.B,
                                     Zero.B,
                                     Read.Capacity.Data,
                                     Msg.Length,
                                     Message,
                                     SCSI.Status,
                                     Execution.Status)
              --}}}  
              --{{{  Prepare Status Message
              Prepare.Status.Message(Execution.Status, Status.Message)
              --}}}  
              --{{{  Convert Capacity info to INT32's
              VAL [2]INT32 RCD RETYPES Read.Capacity.Data:
              SEQ
                Last.Block := Change.Endian.32(RCD[0])
                Block.Size := INT Change.Endian.32(RCD[1])
              --}}}  
              so.write.nl(fs, ts)
              Display.Target.ID.and.LUN(fs, ts, Target.ID, LUN)
              so.write.string(fs, ts, "Number of available blocks = ")
              so.write.int(fs, ts, INT Last.Block, 8)
              so.write.nl(fs, ts)
              so.write.string(fs, ts, "Logical Block Size         = ")
              so.write.int(fs, ts, Block.Size, 8)
              so.write.nl(fs, ts)
              Display.Line(fs, ts, Status.Message)
              Press.Any.Key.To.Continue(fs, ts)
            --}}}  
          4
            --{{{  Read Blocks
            --{{{  Declarations
            TIMER Clock:
            BOOL error:
            BYTE DPO, FUA, RelAdr:
            INT Block.Size, Block.Count:
            INT Logical.Block.Address:
            INT Before.Run, After.Run:
            [80]BYTE Editor.Message:
            --}}}  
            SEQ
              --{{{  Heading
              Display.Line(fs, ts, "Read a Block")
              --}}}  
              --{{{  Initialise
              DPO := Zero.B
              FUA := Zero.B
              RelAdr := Zero.B
              --}}}  
              --{{{  How Many Blocks
              error := TRUE
              WHILE error
                SEQ
                  so.write.string.nl(fs, ts, "How Many Blocks ?")
                  so.read.echo.int(fs, ts, Block.Count, error)
                  so.write.nl(fs, ts)
              --}}}  
              --{{{  Block Size
              error := TRUE
              WHILE error
                SEQ
                  so.write.string.nl(fs, ts, "Block Size ?")
                  so.read.echo.int(fs, ts, Block.Size, error)
                  so.write.nl(fs, ts)
              --}}}  
              --{{{  Logical Block Address
              error := TRUE
              WHILE error
                SEQ
                  so.write.string.nl(fs, ts, "Start reading at logical block address ?")
                  so.read.echo.int(fs, ts, Logical.Block.Address, error)
                  so.write.nl(fs, ts)
              --}}}  
              --{{{  Clear out old data
              initBYTEvec(Data.Block, Zero.B, Block.Count * Block.Size)
              --}}}  
              --{{{  Do the Read
              Set.Identify.Message()
              Clock ? Before.Run
              PRI PAR
                SEQ
                  --{{{  Read 10
                  SCSI.Read.10 ( tscsi,
                                 fscsi,
                                 Target.ID,
                                 LUN,
                                 DPO,
                                 FUA,
                                 RelAdr,
                                 INT32 Logical.Block.Address,
                                 INT32 Block.Count,
                                 INT32 Block.Size,
                                 Non.Linked.Command,
                                 Data.Block,
                                 Msg.Length,
                                 Message,
                                 SCSI.Status,
                                 Execution.Status)
                  --}}}  
                SKIP
              Clock ? After.Run
              --}}}  
              --{{{  Prepare Status Message
              Prepare.Status.Message(Execution.Status, Status.Message)
              --}}}  
              --{{{  Display Results
              IF
                ((INT Execution.Status) = (INT SCSI.E.Good))
                  --{{{  Display Data after successfull read
                  --{{{  Declarations
                  VAL REAL32 Sixty.Four.R32 IS 64.0 (REAL32):
                  INT Time.Taken:
                  REAL32 Time.Taken.R32, Transfer.Size.R32, Mega.Bytes.Per.Second :
                  --}}}  
                  --{{{  Display results
                  [80]BYTE Editor.Message:
                  INT len:
                  SEQ
                    --{{{  Init
                    len := 0
                    SEQ I = 0 FOR SIZE Editor.Message
                      Editor.Message[I] := ' '
                    --}}}  
                    --{{{  Calculate
                    Time.Taken := (After.Run MINUS Before.Run)
                    Time.Taken.R32 := (Sixty.Four.R32 * (REAL32 ROUND Time.Taken))
                    Transfer.Size.R32 := (REAL32 ROUND (Block.Count * Block.Size))
                    Mega.Bytes.Per.Second := Transfer.Size.R32 / Time.Taken.R32
                    --}}}  
                    --{{{  Build Editor Message
                    append.int(len, Editor.Message,Block.Count * Block.Size, 0)
                    append.text(len, Editor.Message, " Bytes Read at ")
                    append.real32(len, Editor.Message, Mega.Bytes.Per.Second, 1, 3)
                    append.text(len, Editor.Message, " MBytes per second.")
                    Centred.Text(Status.Message, Status.Message)
                    --}}}  
                    --{{{  Invoke Hex Editor
                    Hex.Editor( fs, ts, 0, Data.Block, Editor.Message)
                    --}}}  
                  --}}}  
                  --}}}  
                TRUE
                  --{{{  Display Data anyway even though an error has occurred
                  Hex.Editor( fs, ts, 0, Data.Block, Status.Message)
                  --}}}  
              --}}}  
            --}}}  
          5
            --{{{  Write Blocks
            --{{{  Declarations
            TIMER Clock:
            BOOL error:
            BYTE DPO, FUA, RelAdr:
            INT Block.Size, Block.Count:
            INT Logical.Block.Address:
            INT Before.Run, After.Run:
            --}}}  
            SEQ
              --{{{  Heading
              Display.Line(fs, ts, "Write Blocks")
              --}}}  
              --{{{  Initialise
              DPO := Zero.B
              FUA := Zero.B
              RelAdr := Zero.B
              --}}}  
              --{{{  How Many Blocks
              error := TRUE
              WHILE error
                SEQ
                  so.write.string.nl(fs, ts, "How Many Blocks ?")
                  so.read.echo.int(fs, ts, Block.Count, error)
                  so.write.nl(fs, ts)
              --}}}  
              --{{{  Block Size
              error := TRUE
              WHILE error
                SEQ
                  so.write.string.nl(fs, ts, "Block Size ?")
                  so.read.echo.int(fs, ts, Block.Size, error)
                  so.write.nl(fs, ts)
              --}}}  
              --{{{  Logical Block Address
              error := TRUE
              WHILE error
                SEQ
                  so.write.string.nl(fs, ts, "Start writing at logical block address ?")
                  so.read.echo.int(fs, ts, Logical.Block.Address, error)
                  so.write.nl(fs, ts)
              --}}}  
              --{{{  Clear out old data
              --initBYTEvec(Data.Block, Zero.B, Block.Count * Block.Size)
              --}}}  
              --{{{  Display Data
              Hex.Editor( fs, ts, 0, Data.Block, "Edit Data to be Written to Disk")
              --}}}  
              --{{{  Do the Write
              Set.Identify.Message()
              Clock ? Before.Run
              PRI PAR
                SEQ
                  --{{{  Write 10
                  SCSI.Write.10( tscsi,
                                 fscsi,
                                 Target.ID,
                                 LUN,
                                 DPO,
                                 FUA,
                                 RelAdr,
                                 INT32 Logical.Block.Address,
                                 INT32 Block.Count,
                                 INT32 Block.Size,
                                 Non.Linked.Command,
                                 Data.Block,
                                 Msg.Length,
                                 Message,
                                 SCSI.Status,
                                 Execution.Status)
                  --}}}  
                SKIP
              Clock ? After.Run
              --}}}  
              --{{{  Check for errors
              Prepare.Status.Message(Execution.Status, Status.Message)
              --}}}  
              --{{{  Display Transfer rate if transfer was OK
              --{{{  Prepare Status Message
              Prepare.Status.Message(Execution.Status, Status.Message)
              --}}}  
              --{{{  Display Results
              IF
                ((INT Execution.Status) = (INT SCSI.E.Good))
                  --{{{  Declarations
                  VAL REAL32 Sixty.Four.R32 IS 64.0 (REAL32):
                  INT Time.Taken:
                  REAL32 Time.Taken.R32, Transfer.Size.R32, Mega.Bytes.Per.Second :
                  --}}}  
                  --{{{  Display results
                  [80]BYTE Editor.Message:
                  INT len:
                  SEQ
                    --{{{  init
                    len := 0
                    SEQ I = 0 FOR SIZE Editor.Message
                      Editor.Message[I] := ' '
                    --}}}  
                    --{{{  Calculate
                    Time.Taken := (After.Run MINUS Before.Run)
                    Time.Taken.R32 := (Sixty.Four.R32 * (REAL32 ROUND Time.Taken))
                    Transfer.Size.R32 := (REAL32 ROUND (Block.Count * Block.Size))
                    Mega.Bytes.Per.Second := Transfer.Size.R32 / Time.Taken.R32
                    --}}}  
                    --{{{  Build Editor Message
                    append.int(len, Editor.Message,Block.Count * Block.Size, 0)
                    append.text(len, Editor.Message, " Bytes Read at ")
                    append.real32(len, Editor.Message, Mega.Bytes.Per.Second, 1, 3)
                    append.text(len, Editor.Message, " MBytes per second.")
                    --}}}  
                    --{{{  Display results
                    Display.Line(fs, ts, Editor.Message)
                    Press.Any.Key.To.Continue(fs, ts)
                    --}}}  
                  --}}}  
                TRUE
                  SKIP
              --}}}  
              --}}}  
            --}}}  
          6
            Not.Implemented(fs, ts)
          7
            --{{{  Start Stop
            SEQ
              Set.Identify.Message()
              Display.Line(fs, ts, "Starting Drive")
            
              SCSI.Start.Stop.Unit.6 (tscsi,
                                      fscsi,
                                      Target.ID,
                                      LUN,
                                      1 (BYTE), -- Not immediate
                                      0 (BYTE), -- Start up drive
                                      Non.Linked.Command,
                                      Msg.Length,
                                      Message,
                                      SCSI.Status,
                                      Execution.Status)
            
            
              Press.Any.Key.To.Continue(fs, ts)
            --}}}  
          8
            --{{{  Seek
            --{{{  Declarations
            INT Logical.Block.Address:
            BOOL error:
            --}}}  
            SEQ
              --{{{  Heading
              Display.Line(fs, ts, "Seek to a Block")
              --}}}  
              --{{{  Logical Block Address
              error := TRUE
              WHILE error
                SEQ
                  so.write.string.nl(fs, ts, "Seek to logical block address ?")
                  so.read.echo.int(fs, ts, Logical.Block.Address, error)
                  so.write.nl(fs, ts)
              --}}}  
              --{{{  Do the Seek
              Set.Identify.Message()
              
              SCSI.Seek.10 ( tscsi,
                             fscsi,
                             Target.ID,
                             LUN,
                             INT32 Logical.Block.Address,
                             Non.Linked.Command,
                             Msg.Length,
                             Message,
                             SCSI.Status,
                             Execution.Status)
              --}}}  
              --{{{  Prepare Status Message
              Prepare.Status.Message(Execution.Status, Status.Message)
              --}}}  
            --}}}  
          9
            Format.Options(fs, ts, tscsi, fscsi)
          ELSE
            so.write.string.nl(fs, ts, "Not Implemented")
        --}}}  
  --}}}  
:
