/*
 * Copyright (c) INMOS Limited 1990
 *
 * Description  : Constant definitions for the WITCH IMS B422 SCSI TRAM
 *                driver.
 *
 * Date         : 05-Nov-90
 *
 * Filename     : b422def.h
 *
 * Project      : WITCH
 *
 * Author       : Mike Burrow
 *
 * Version      : 2.0
 *
 * Division     : CAD
 *
 * Limitations  : None
 *
 * Amendments   :
 *
 */


/* 
 * Definition of occam-like types
 */
typedef short   INT16;
typedef int     INT32;
typedef int     INT64;
typedef byte    BOOL;

/*
 * Sizes
 */
#define SCSI_6                                  0x0006
#define SCSI_10                                 0x000a
#define NULL_LENGTH                             0x0000
#define NULL_SIZE                               0x0001
#define CAPACITY_DATA_LENGTH                    0x0008
#define REQUEST_SENSE_DATA_LENGTH               0x0012

/* 
 * SCSI command arrays 
 */
typedef BYTE    SCSI_CMND_6[SCSI_6];
typedef BYTE    SCSI_CMND_10[SCSI_10];
typedef BYTE    NULL_BUFFER[NULL_SIZE];


/*
 * Software ID
 */
#define SCSI_SOFTWARE_ID                0x0001

/*
 * I_DIRECTION
 */
#define NON_DATA_TRANSFER               0x0000
#define HOST_TO_SCSI                    0x0001
#define SCSI_TO_HOST                    0x0002
#define TARGET_TO_SCSI                  0x0003
#define SCSI_TO_TARGET                  0x0004
#define MAX_SCSI_VALID_DIRECTION        0x0004

/*
 * General
 */
#define ZERO_B                          0x0000
#define ZERO_16                         0x0000
#define ZERO_32                         0x0000
#define NEGATED_B                       0x0000
#define ASSERTED_B                      0x00FF
#define LOW_BYTE                        0x00ff
#define HIGH_BYTE                       0xff00
#define LP_1_SECOND                     0x3d09

/*
 * Control Byte
 */
#define NON_LINKED_COMMAND              ZERO_B

/*
 * Operating Mode
 */
#define INITIATOR                       0x0000
#define TARGET                          0x0001

/*
 * Data Transfer Mode
 */
#define SCSI_FAST_DMA_MODE              0x0001
#define SCSI_SLOW_DMA_MODE              0x0002
#define SCSI_VERY_SLOW_DMA_MODE         0x0003

/*
 * Link Management
 */
#define SINGLE_LINK                     0x0000
#define DOUBLE_LINK_L_PLUS_1            0x0001
#define DOUBLE_LINK_L_PLUS_2            0x0002
#define DOUBLE_LINK_L_PLUS_3            0x0003
#define MAX_LINK_MESSAGE_SIZE           0x0100
#define LINK_COUNT                      0x0004

/*
 * Bus Connection State
 */
#define DISCONNECTED_STATE              ZERO_B
#define CONNECTED_STATE                 0x0001

/*
 * Buffer Sizes
 */
#define MAX_SCSI_BUFFER_SIZE            0x4000
#define MIN_SCSI_BUFFER_SIZE            0x0100

/*
 * Packet Management
 */
#define MAX_SCSI_PACKET_SIZE            0x2000
#define MIN_SCSI_PACKET_SIZE            0x0100

/*
 * SCSI Protocol Sizes
 */
#define MAX_SCSI_COMMAND_SIZE           0x0100
#define MAX_SCSI_MESSAGE_SIZE           0x0100
#define MAX_SCSI_IO_PROCESSES           0x0100
#define MAX_SCSI_LUN                    0x0007
#define MAX_SCSI_DEVICE_ID              0x0007

/*
 * Watch Dog Timer Defaults
 */
#define DEFAULT_TICKS_PER_LOOP          0x3d09
#define DEFAULT_DATA_PHASE_TIME_OUT     0x000a
#define DEFAULT_INTERRUPT_TIME_OUT      0x03e8

/*
 * Initilisation
 */
#define ENABLE_PARITY_CHECKING          ASSERTED_B
#define DISABLE_PARITY_CHECKING         ZERO_B
#define ENABLE_PARITY_GENERATION        ASSERTED_B
#define DISABLE_PARITY_GENERATION       ZERO_B
#define FAST_CABLE_MODE                 ASSERTED_B
#define SLOW_CABLE_MODE                 ZERO_B
#define RESET_SCSI_CONTROLLER           ASSERTED_B
#define DO_NOT_RESET_SCSI_CONTROLLER    ZERO_B
#define RESET_SCSI_BUS                  ASSERTED_B
#define DO_NOT_RESET_SCSI_BUS           ZERO_B
#define DO_SELF_TEST                    ASSERTED_B
#define DO_NOT_SELF_TEST                ZERO_B

/* 
 * PROTOCOL TO.SCSI
 */
#define SCSI_BECOME_AN_INITIATOR             0x00
#define SCSI_BECOME_TARGET                   (SCSI_BECOME_AN_INITIATOR + 1)
#define SCSI_I_TARGET_ID                     (SCSI_BECOME_TARGET + 1)
#define SCSI_I_LUN                           (SCSI_I_TARGET_ID + 1)
#define SCSI_I_MESSAGE_BUFFER                (SCSI_I_LUN + 1)
#define SCSI_I_COMMAND_BUFFER                (SCSI_I_MESSAGE_BUFFER + 1)
#define SCSI_I_DIRECTION                     (SCSI_I_COMMAND_BUFFER + 1)
#define SCSI_I_START_IO_PROCESS              (SCSI_I_DIRECTION + 1)
#define SCSI_I_TX_LENGTH                     (SCSI_I_START_IO_PROCESS + 1)
#define SCSI_I_RX_LENGTH                     (SCSI_I_TX_LENGTH + 1)
#define SCSI_I_TX_DATA                       (SCSI_I_RX_LENGTH + 1)
#define SCSI_T_AWAIT_SELECTION_BY_INITIATOR  (SCSI_I_TX_DATA + 1)
#define SCSI_T_DISCONNECT                    (SCSI_T_AWAIT_SELECTION + 1)
#define SCSI_T_RECONNECT                     (SCSI_T_DISCONNECT + 1)
#define SCSI_T_DATA_IN                       (SCSI_T_RECONNECT + 1)
#define SCSI_T_DATA_OUT                      (SCSI_T_DATA_IN + 1)
#define SCSI_T_TX_DATA                       (SCSI_T_DATA_OUT + 1)
#define SCSI_T_STATUS                        (SCSI_T_TX_DATA + 1)
#define SCSI_T_MESSAGE_IN                    (SCSI_T_STATUS + 1)
#define SCSI_T_MESSAGE_OUT                   (SCSI_T_MESSAGE_IN + 1)
#define SCSI_T_STATUS_AND_MESSAGE_IN         (SCSI_T_MESSAGE_OUT + 1)
#define SCSI_S_DEFINE_ID                     (SCSI_T_STATUS_AND_MESSAGE_IN + 1)
#define SCSI_S_RESET_SCSI_BUS                (SCSI_S_DEFINE_ID + 1)
#define SCSI_S_RESET_SCSI_ADAPTOR            (SCSI_S_RESET_SCSI_BUS + 1)
#define SCSI_DATA_TRANSFER_MODE              (SCSI_S_RESET_SCSI_ADAPTOR + 1)
#define SCSI_LINK_MANAGEMENT                 (SCSI_DATA_TRANSFER_MODE + 1)
#define SCSI_PM_REQUEST_PACKET_SIZE          (SCSI_LINK_MANAGEMENT + 1)
#define SCSI_MESSAGE_TO_OTHER_LINK           (SCSI_PM_REQUEST_PACKET_SIZE + 1)
#define SCSI_SS_RESET_SUBSYSTEM              (SCSI_MESSAGE_TO_OTHER_LINK + 1)
#define SCSI_SS_ANALYSE_SUBSYSTEM            (SCSI_SS_RESET_SUBSYSTEM + 1)
#define SCSI_SS_READ_SUBSYSTEM_NOTERROR      (SCSI_SS_ANALYSE_SUBSYSTEM + 1)
#define SCSI_PERFORM_SELF_TEST               (SCSI_SS_READ_SUBSYSTEM_NOTERROR + 1)
#define SCSI_SID_REPORT_VERSION_NUMBER       (SCSI_PERFORM_SELF_TEST + 1)
#define SCSI_SID_REPORT_VENDOR               (SCSI_SID_REPORT_VERSION_NUMBER + 1)
#define SCSI_E_REQUEST_EXECUTION_STATUS      (SCSI_SID_REPORT_VENDOR + 1)
#define SCSI_WD_DATA_PHASE_TIME_OUT          (SCSI_E_REQUEST_EXECUTION_STATUS + 1)
#define SCSI_WD_INTERRUPT_TIME_OUT           (SCSI_WD_DATA_PHASE_TIME_OUT + 1)
#define GENERIC_INPUT                        (SCSI_WD_INTERRUPT_TIME_OUT + 1)

/*
 * PROTOCOL FROM.SCSI
 */
#define SCSI_I_TX_SEND_PACKET           0x00
#define SCSI_I_RX_DATA                  (SCSI_I_TX_SEND_PACKET + 1)
#define SCSI_I_MSGBUFF                  (SCSI_I_RX_DATA + 1)
#define SCSI_I_STATUS                   (SCSI_I_MSGBUFF + 1)
#define SCSI_T_COMMAND_BUFFER           (SCSI_I_STATUS + 1)
#define SCSI_T_MSG_OUT_DATA             (SCSI_T_COMMAND_BUFFER + 1)
#define SCSI_T_RX_DATA                  (SCSI_T_MSG_OUT_DATA + 1)
#define SCSI_PM_ALLOCATED_PACKET_SIZE   (SCSI_T_RX_DATA + 1)
#define SCSI_MESSAGE_FROM_OTHER_LINK    (SCSI_PM_ALLOCATED_PACKET_SIZE + 1)
#define SCSI_E_EXECUTION_STATUS         (SCSI_MESSAGE_FROM_OTHER_LINK + 1)
#define SCSI_DL_END_DOUBLE_LINK         (SCSI_E_EXECUTION_STATUS + 1)
#define SCSI_SS_SUBSYSTEM_NOTERROR      (SCSI_DL_END_DOUBLE_LINK + 1)
#define SCSI_SID_VERSION_NUMBER         (SCSI_SS_SUBSYSTEM_NOTERROR + 1)
#define SCSI_SID_VENDOR                 (SCSI_SID_VERSION_NUMBER + 1)
#define SCSI_GENERIC_OUTPUT             (SCSI_SID_VENDOR + 1)

/*
 * B422 SCSI ID 
 */
#define B422_SCSI_ID                    0x0007


/* 
 * Link Constants For Helios 
 */
#define MAX_LINK_CAPACITY               0x3000
#define LINKTIMEOUT                     0x40000000


/* 
 * SCSI commands
 */

#define TEST_UNIT_READY         0x000
#define REZERO_UNIT             0x001
#define REQUEST_SENSE           0x003
#define FORMAT_UNIT             0x004
#define REASSIGN_BLOCKS         0x007
#define READ                    0x008
#define WRITE                   0x00a
#define SEEK                    0x00b
#define READ_USAGE_COUNTER      0x011
#define INQUIRY                 0x012
#define MODE_SELECT             0x015
#define RESERVE                 0x016
#define RELEASE                 0x017
#define MODE_SENSE              0x01a
#define START_STOP              0x01b
#define RECIEVE_DIAG_RESULTS    0x01c
#define SEND_DIAGNOSTIC         0x01d
#define READ_CAPACITY           0x025
#define READ_10                 0x028
#define WRITE_10                0x02a
#define SEEK_10                 0x02b
#define VERIFY                  0x02f
#define READ_DEFECT_DATA        0x037
#define WRITE_BUFFER            0x03b
#define READ_BUFFER             0x03c
#define READ_LONG               0x0e5
#define WRITE_LONG              0x0e6


/*
 * Error codes.
 */
#define SCSI_E_GOOD                                     0x0000
#define SCSI_E_BAD_SCSI_STATUS                          0x0001
#define SCSI_E_COMMAND_NOT_COMPLETE                     0x0002
#define SCSI_E_INCORRECT_IO_PROCESS_NUMBER              0x0003
#define SCSI_E_SCSI_BUS_PARITY_ERROR                    0x0004
#define SCSI_E_DATA_IN_FAILURE                          0x0005
#define SCSI_E_BAD_DATA_IN_RETURNED_PACKET_SIZE         0x0006
#define SCSI_E_INCORRECT_IO_PNO_DURING_DATA_IN          0x0007
#define SCSI_E_DATA_OUT_FAILURE                         0x0008
#define SCSI_E_DATA_IN_TIME_OUT                         0x0009
#define SCSI_E_DATA_OUT_TIME_OUT                        0x000a
#define SCSI_E_COMMAND_TIME_OUT                         0x000b
#define SCSI_E_STATUS_TIME_OUT                          0x000c
#define SCSI_E_CONTROLLER_RESET_TIME_OUT                0x000d
#define SCSI_E_BUS_RESET_TIME_OUT                       0x000e
#define SCSI_E_RESELLECTED                              0x000f
#define SCSI_E_FUNCTION_NOT_COMPLETE                    0x0010
#define SCSI_E_DISCONNECTED                             0x0011
#define SCSI_E_NOT_BUS_SERVICE                          0x0012
#define SCSI_E_SEQUENCE_NOT_COMPLETE                    0x0013
#define SCSI_E_PREMATURE_PHASE_CHANGE                   0x0014
#define SCSI_E_TARGET_DID_NOT_ASSERT_ATN                0x0015
#define SCSI_E_TARGET_DID_NOT_ASSERT_MESSAGE_OUT        0x0016
#define SCSI_E_SELECTION_TIME_OUT                       0x0017
#define SCSI_E_UNDOCUMENTED_ERROR                       0x0018
#define SCSI_E_DATA_IN_ABORT                            0x0019
#define SCSI_E_DATA_OUT_ABORT                           0x001a
#define SCSI_E_UNEXPECTED_PHASE_CHANGE                  0x001b
#define SCSI_E_FAILED_SELF_TEST                         0x001c
#define SCSI_E_INVALID_INTERVAL_OFFSET                  0x001d
#define SCSI_E_BUS_RESET                                0x001e
#define SCSI_E_BAD_MESSAGE_LENGTH                       0x001f
#define SCSI_E_BAD_COMMAND_LENGTH                       0x0020
#define SCSI_E_BAD_LUN                                  0x0021
#define SCSI_E_BAD_DEVICE_ID                            0x0022
#define SCSI_E_INVALID_DATA_TRANSFER_SIZE               0x0023
#define SCSI_E_INTERRUPT_TIME_OUT                       0x0024
#define SCSI_E_BAD_DATA_PHASE_TIME_OUT                  0x0025
#define SCSI_E_BAD_INTERRUPT_TIME_OUT                   0x0026
#define SCSI_E_SOFTWARE_ERROR                           0x07fe
#define SCSI_E_NOT_IMPLEMENTED                          0x07ff
