--{{{  Header
--#COMMENT "Copyright INMOS Limited 1990, 1991, 1992"
--#COMMENT "IMS F002C Product release SCSI software libraries"
--------------------------------------------------------------------------------
--
-- Author      : Paul Fellows
--
-- Date        : 19-Jan-1990 11:43:49
--
-- File Name   : NCR53C94.INC
--
-- Project     : IMS F002C - SCSI Tram Software
--
-- Version     : 0.1
--
-- Division    : IQ Systems
--
-- Limitations : This software may be distributed in source form.
--
-- Description : Definitions for NCR 53c94 SCSI Controller chip
--
--------------------------------------------------------------------------------
-- Schedule of Amendments
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--}}}  
--{{{  Declarations
--{{{  Commands
--The NCR 53c94 responds to 26 commands, by which the host can initiate
--data transfers, prepare the controller for selection, etc- They are as
--follows and are issued to the Command Register :-
--{{{  Miscellaneous Group
--{{{  Non DMA
VAL Nop                                          IS #0000 :
VAL Flush.FIFO                                   IS #0001 :
VAL Reset.Chip                                   IS #0002 :
VAL Reset.SCSI.Bus                               IS #0003 :
--}}}  
--{{{  DMA
VAL Nop.DMA                                      IS #0080 :
VAL DMA.Nop                                      IS #0080 :
VAL Flush.FIFO.DMA                               IS #0081 :
VAL Reset.Chip.DMA                               IS #0082 :
VAL Reset.SCSI.Bus.DMA                           IS #0083 :
--}}}  
--}}}  
--{{{  Intiator State Group
--{{{  Non DMA
VAL Transfer.Information                         IS #0010 :
VAL Initiator.Command.Complete.Sequence          IS #0011 :
VAL Message.Accepted                             IS #0012 :
VAL Transfer.Pad                                 IS #0018 :
VAL Set.Atn                                      IS #001A :
--}}}  
--{{{  DMA
VAL Transfer.Information.DMA                     IS #0090 :
VAL Initiator.Command.Complete.Sequence.DMA      IS #0091 :
VAL Message.Accepted.DMA                         IS #0092 :
VAL Transfer.Pad.DMA                             IS #0098 :
VAL Set.Atn.DMA                                  IS #009A :
--}}}  
--}}}  
--{{{  Target State Group
--{{{  Non DMA
VAL Send.Message                                 IS #0020 :
VAL Send.Status                                  IS #0021 :
VAL Send.Data                                    IS #0022 :
VAL Disconnect.Sequence                          IS #0023 :
VAL Terminate.Sequence                           IS #0024 :
VAL Target.Command.Complete.Sequence             IS #0025 :
VAL Disconnect.From.Bus                          IS #0027 :
VAL Receive.Message.Sequence                     IS #0028 :
VAL Receive.Command                              IS #0029 :
VAL Receive.Data                                 IS #002A :
VAL Receive.Command.Sequence                     IS #002B :
VAL Target.Abort.DMA                             IS #0004 :
--}}}  
--{{{  DMA
VAL Send.Message.DMA                             IS #00A0 :
VAL Send.Status.DMA                              IS #00A1 :
VAL Send.Data.DMA                                IS #00A2 :
VAL Disconnect.Sequence.DMA                      IS #00A3 :
VAL Terminate.Sequence.DMA                       IS #00A4 :
VAL Target.Command.Complete.Sequence.DMA         IS #00A5 :
VAL Disconnect.From.Bus.DMA                      IS #00A7 :
VAL Receive.Message.Sequence.DMA                 IS #00A8 :
VAL Receive.Command.DMA                          IS #00A9 :
VAL Receive.Data.DMA                             IS #00AA :
VAL Receive.Command.Sequence.DMA                 IS #00AB :
VAL Target.Abort.DMA.DMA                         IS #0084 :
--}}}  
--}}}  
--{{{  Disconnected State Group
--{{{  Non DMA
VAL Reselect.Sequence                            IS #0040 :
VAL Select.Without.Atn.Sequence                  IS #0041 :
VAL Select.With.Atn.Sequence                     IS #0042 :
VAL Select.With.Atn.And.Stop.Sequence            IS #0043 :
VAL Enable.Selection.Or.Reselection              IS #0044 :
VAL Disable.Selection.Or.Reselection             IS #0045 :
VAL Select.with.ATN3                             IS #0046 :
--}}}  
--{{{  DMA
VAL Reselect.Sequence.DMA                        IS #00C0 :
VAL Select.Without.Atn.Sequence.DMA              IS #00C1 :
VAL Select.With.Atn.Sequence.DMA                 IS #00C2 :
VAL Select.With.Atn.And.Stop.Sequence.DMA        IS #00C3 :
VAL Enable.Selection.Or.Reselection.DMA          IS #00C4 :
VAL Disable.Selection.Or.Reselection.DMA         IS #00C5 :
VAL Select.with.ATN3.DMA                         IS #00C6 :
--}}}  
--}}}  
--}}}  
--{{{  Mask Definitions
--{{{  Command Register
VAL DMA                                          IS #0080 :
VAL NCR.Command                                  IS #007F :
--}}}  
--{{{  Status Register
--
--The Status register is an 8 bit read only register which contain fields
--to indicate the status of the chip and to qualify the reason for an
--interrupt.

VAL I.O.Mask                                     IS #0001 :
VAL C.D.Mask                                     IS #0002 :
VAL MSG.Mask                                     IS #0004 :
VAL SCSI.Bus.Phase.Mask                          IS #0007 :
VAL Valid.Group.Code.Mask                        IS #0008 :
VAL Terminal.Count.Mask                          IS #0010 :
VAL Parity.Error.Mask                            IS #0020 :
VAL Gross.Error.Mask                             IS #0040 :
VAL Interrupt.Mask                               IS #0008 :
--}}}  
--{{{  Interrupt Register
--The interrupt status register is an 8 bit read only register that used
--in conjunction with the status register and sequence step register
--(see later) determines the cause of an interrupt.

VAL Selected.Mask                                IS #0001 :
VAL Selected.With.Atn.Mask                       IS #0002 :
VAL Reselected.Mask                              IS #0004 :
VAL Function.Complete.Mask                       IS #0008 :
VAL Bus.Service.Mask                             IS #0010 :
VAL Disconnect.Mask                              IS #0020 :
VAL Illegal.Command.Mask                         IS #0040 :
VAL SCSI.Reset.Detected.Mask                     IS #0080 :
--}}}  
--{{{  Time Out Register
--
--The Time-Out register is an eight bit write only register which specifies
--the amount of time to wait for a response during selection or Re-selection.
--
-- No mask required as all 8 bits form part of the same data structure.
--}}}  
--{{{  Sequence Step Register
--
--The Sequence step register is an eight bit read only register which
--indicates the current substep within certain ASC sequences when an
--interrupt occurs. Only the least significant nibble is valid.

VAL SS0.Mask                                     IS #0001 :
VAL SS1.Mask                                     IS #0002 :
VAL SS2.Mask                                     IS #0004 :
VAL Not.SOM.Mask                                 IS #0008 :
VAL Sequence.Step.Mask                           IS #0007 :
VAL Arbitration.Complete                         IS #0000 :
VAL Expected.Command.Phase                       IS #0002 :
VAL Premature.Phase.Change                       IS #0003 :
VAL Select.With.ATN.Sequence.Complete            IS #0004 :
VAL Select.Sequence.Complete                     IS #0004 :
--}}}  
--{{{  Synchronous Transfer Period Register
--
--The Synchronous Transfer Period register is an eight bit write only register
--of which the least significant 5 bits specify the minimum time between
--leading edges of successive REQuest or ACKnowledge pulses.

VAL Synchronous.Transfer.Period.Mask             IS #001F :
--}}}  
--{{{  FIFO Flags Register
--
--The FIFO Flags register is an eight bit write only register of which the
--least significant 5 bits indicate how many bytes are currently in the FIFO.
--Use only for post mortems as it is not guarenteed to be stable whilst a
--transfer is taking place. The most sigificant 3 bits are duplicates of the
--Sequence Step register in normal mode, but if Test Mode is enabled, bit 5 is
--set to indicate that the offset counter is not zero.

VAL FIFO.Flags.Mask                              IS #001F :
VAL FIFO.Flags.Sequence.Step.Mask                IS #00E0 :
--}}}  
--{{{  Synchronous Offset Register
--
--The Synchronous Offset register is an eight bit write only register of which
--the least signoficant 4 bits specify whether the ASC will transfer Data
--Phase bytes synchronously ar asynchronously

VAL Synchronous.Offset.Mask                      IS #000F :
--}}}  
--{{{  Configuration Register
--
--The configuration-1 register is an 8 bit read/write register which is used
--to specify the different operating options for the ASC

VAL Bus.Id.Mask                                  IS #0007 :
VAL Chip.Test.Mode.Mask                          IS #0080 :
VAL Enable.Parity.Checking.Mask                  IS #0010 :
VAL Parity.Test.Mode.Mask                        IS #0020 :
VAL SCSI.Reset.Int.Disable.Mask                  IS #0040 :
VAL Slow.Cable.Mode.Mask                         IS #0080 :
--}}}  
--{{{  Test Register
--
--The Test Register is an 8 bit Write only register which is used to define
--chip test mode.

VAL Target.Mode.Mask                             IS #0001 :
VAL Initiator.Mode.Mask                          IS #0002 :
VAL Tri.State.Mode.Mask                          IS #0004 :
--}}}  
--{{{  Configuration 2 Register
--
--The configuration-2 register is an 8 bit read/write register which is used
--to specify the different operating options for the ASC.

VAL DMA.Parity.Enable.Mask                       IS #0001 :
VAL Register.Parity.Enable.Mask                  IS #0002 :
VAL Target.Bad.Partity.Abort.Mask                IS #0004 :
VAL SCSI.2.Mask                                  IS #0008 :
VAL DREQ.High.Impedance.Mask                     IS #0010 :
VAL Enable.Byte.Control.Mask                     IS #0020 :
VAL Enable.Phase.Latch.Mask                      IS #0040 :
VAL Reserve.FIFO.Byte.Mask                       IS #0080 :
--}}}  
--{{{  Configuration 3 Register
--
--The configuration-3 register is an 8 bit read/write register which is used
--to specify the different operating options for the ASC.

VAL Normal.DMA.Mode                              IS #0000 :
VAL Threshold.8.Mask                             IS #0001 :
VAL Alternate.DMA.Mode.Mask                      IS #0002 :
VAL Threshold.8.and.Alternate.DMA.Mode           IS #0003 :
VAL Save.Residual.Byte.Mask                      IS #0004 :
--}}}  
--}}}  
--{{{  Addresses
--{{{  Base Address
-- define a base address for the controller and make all register addresses
-- relative to it to ease portability. See B422hwdf.inc
--}}}  
--{{{  Read Only Addresses
VAL Status.Address                               IS Ncr53c94.Base.Address + #04:
VAL Interrupt.Address                            IS Ncr53c94.Base.Address + #05:
VAL Sequence.Step.Address                        IS Ncr53c94.Base.Address + #06:
VAL FIFO.Flags.Address                           IS Ncr53c94.Base.Address + #07:
VAL NCR.Reserved.1.Address                       IS Ncr53c94.Base.Address + #09:
VAL NCR.Reserved.2.Address                       IS Ncr53c94.Base.Address + #0A:
--}}}  
--{{{  Read/Write Addresses
VAL Transfer.Count.LSB.Address                   IS Ncr53c94.Base.Address + #00:
VAL Transfer.Count.MSB.Address                   IS Ncr53c94.Base.Address + #01:
VAL FIFO.Address                                 IS Ncr53c94.Base.Address + #02:
VAL Command.Address                              IS Ncr53c94.Base.Address + #03:
VAL Configuration.1.Address                      IS Ncr53c94.Base.Address + #08:
VAL Configuration.2.Address                      IS Ncr53c94.Base.Address + #0B:
VAL Configuration.3.Address                      IS Ncr53c94.Base.Address + #0C:
--}}}  
--{{{  Write Only Addresses
VAL Destination.Bus.ID.Address                   IS Ncr53c94.Base.Address + #04:
VAL Select.Reselect.Timeout.Address              IS Ncr53c94.Base.Address + #05:
VAL Synchronous.Period.Address                   IS Ncr53c94.Base.Address + #06:
VAL Synchronous.Offset.Address                   IS Ncr53c94.Base.Address + #07:
VAL Clock.Conversion.Factor.Address              IS Ncr53c94.Base.Address + #09:
VAL Test.Mode.Address                            IS Ncr53c94.Base.Address + #0A:
--}}}  
--}}}  
--{{{  Constant definitions
VAL INT32 ESP.Transfer.Max                    IS #10000 (INT32):
VAL INT Default.Select.Reselect.Timeout.Value IS #0099:
VAL INT Default.Synchronous.Offset.Value      IS #0000:
VAL INT Default.Synchronous.Period.Value      IS #0000:
VAL INT Default.Clock.Conversion.Factor.Value IS #0005:
VAL INT Default.Reset.Selection.Time.Delay    IS 3907 : -- 250 mSeconds
--}}}  
--}}}  
