--{{{  Header
--#COMMENT "Copyright INMOS Limited 1990, 1991, 1992"
--#COMMENT "IMS F002C Product release SCSI software libraries"
--------------------------------------------------------------------------------
--
-- Date        : 19-Jan-1990 09:46:20
--
-- File Name   : txndmalp.INC
--
-- Project     : F002 - SCSI Tram Software
--
-- Version     : 1.0
--
-- Division    : IQ Systems
--
-- Limitations : This software may be distributed in source form.
--
-- Description : An rx non dma transfer loop. ie one byte at a time via the
--               MPU interface. REAL SLOW !!!!!!!
--
--------------------------------------------------------------------------------
-- Schedule of Amendments
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--
--}}}  
--{{{  implementation
PROC Rx.Non.DMA.Loop( VAL INT            Interrupt.Time.Out,
                      VAL INT            Packet.Size,
                      VAL INT            Data.Transfer.Mode,  -- -1..-32768
                      VAL INT            Rx.DMA.Buffer.Pointer,
                      [3]INT             NCR.Status,
                      INT16              Execution.Status)

  --{{{  Uses
  #USE "b422irpt.lib"
  #USE "disconct.lib"
  --#USE "debug.lib"
  --}}}  
  --{{{  Declarations
  INT Byte.Count :
  INT Transfer.Mode:
  --}}}  
  --{{{  Implementation
  SEQ
    --{{{  Declarations
    IF
      (Data.Transfer.Mode = (MOSTNEG INT))
        Transfer.Mode := MOSTPOS INT
      TRUE
        Transfer.Mode := - Data.Transfer.Mode -- Make Negative value positive.
    
    Byte.Count := 0
    []BYTE Rx.Bytes RETYPES [Rx.Buffer FROM Rx.DMA.Buffer.Pointer FOR Packet.Size]:
    --}}}  
    --{{{  Perform Non DMA
    IF
      Transfer.Mode = 1
        --{{{  Non DMA 1 byte at a time with no delay
        SEQ
          WHILE Byte.Count < Packet.Size
            SEQ
              --{{{  Transfer a Byte
              Command := Transfer.Information  -- Non DMA
              --{{{  Wait for interrupt
              IF
                (Execution.Status = SCSI.E.Good)
                  SEQ
                    Accept.Interrupt ( WAIT.FOR.INTERRUPT,
                                       Interrupt.Time.Out,
                                       NCR.Status,
                                       Execution.Status)
                    --{{{  Interrogate
                    --{{{  Check for SCSI Bus Reset
                    IF
                      ((NCR.Status[Interrupt.Pointer] /\ SCSI.Reset.Detected.Mask) =
                      SCSI.Reset.Detected.Mask)
                        IF
                          Execution.Status = SCSI.E.Good
                            Execution.Status := SCSI.E.Bus.Reset
                          TRUE
                            SKIP
                      TRUE
                        --{{{  Interrogate registers
                        IF
                          ((NCR.Status[Status.Pointer] /\ Parity.Error.Mask) = Parity.Error.Mask)
                            IF
                              Execution.Status = SCSI.E.Good
                                Execution.Status := SCSI.E.SCSI.Bus.Parity.Error
                              TRUE
                                SKIP
                          TRUE
                            IF
                              Execution.Status = SCSI.E.Good
                                --{{{  Interrogate further
                                SEQ
                                  SCSI.Phase :=  (NCR.Status[Status.Pointer] /\ SCSI.Bus.Phase.Mask)
                                  IF
                                    ((NCR.Status[Interrupt.Pointer] /\ Disconnect.Mask) = Disconnect.Mask)
                                      --{{{  Disconnect Handler
                                      BOOL Finished:
                                      INT Next.SCSI.Phase:
                                      BYTE Msg.Length:
                                      [1]BYTE Message:
                                      SEQ
                                        Finished := FALSE
                                        Execution.Status := SCSI.E.Disconnected
                                        WHILE NOT Finished
                                          SEQ
                                            --{{{  Disconnect Handler
                                            SCSI.I.Disconnect.Handler( WAIT.FOR.INTERRUPT,
                                                                       Interrupt.Time.Out,
                                                                       SCSI.Phase,
                                                                       NCR.Status,
                                                                       Execution.Status)
                                            --}}}  
                                            --{{{  Execute phase requested by target
                                            CASE INT Next.SCSI.Phase
                                              Status.Phase
                                                SEQ
                                                  Execution.Status := SCSI.E.Unexpected.Phase.Change
                                              Data.In.Phase
                                                SEQ
                                                  Finished := TRUE
                                              Data.Out.Phase
                                                SEQ
                                                  Execution.Status := SCSI.E.Unexpected.Phase.Change
                                              Command.Phase
                                                SEQ
                                                  Execution.Status := SCSI.E.Unexpected.Phase.Change
                                              Message.Out.Phase
                                                SEQ
                                                  --{{{  Message Out Phase
                                                  SCSI.I.Message.Out.Phase(WAIT.FOR.INTERRUPT,
                                                                           Interrupt.Time.Out,
                                                                           Msg.Length,
                                                                           Message,
                                                                           Next.SCSI.Phase,
                                                                           NCR.Status,
                                                                           Finished,
                                                                           Execution.Status)
                                                  Finished := TRUE
                                                  --}}}  
                                              Message.In.Phase
                                                SEQ
                                                  --{{{  Message IN Phase
                                                  SCSI.I.Message.In.Phase ( WAIT.FOR.INTERRUPT,
                                                                            Interrupt.Time.Out,
                                                                            Next.SCSI.Phase,
                                                                            Msg.Length,
                                                                            Message,
                                                                            NCR.Status,
                                                                            Execution.Status)
                                                  --}}}  
                                              ELSE
                                                --{{{  Software Error
                                                IF
                                                  Execution.Status = SCSI.E.Good
                                                    SEQ
                                                      Execution.Status := SCSI.E.Software.Error
                                                  TRUE
                                                    SKIP
                                                --}}}  
                                            --}}}  
                                      --}}}  
                                    TRUE
                                      IF
                                        (Operating.Mode = Initiator)
                                          IF
                                            ((NCR.Status[Interrupt.Pointer] /\ Bus.Service.Mask) <> Bus.Service.Mask)
                                              SEQ
                                                Execution.Status := SCSI.E.Good --SCSI.E.Data.In.Failure
                                            TRUE
                                              SKIP
                                        (Operating.Mode = Target)
                                          SKIP
                                        TRUE
                                          Execution.Status := SCSI.E.Software.Error
                                --}}}  
                              TRUE
                                SKIP
                        --}}}  
                    --}}}  
                    --}}}  
                TRUE
                  SKIP
              --}}}  
              --{{{  Access FIFO
              Rx.Bytes[Byte.Count] := BYTE (FIFO /\ Low.Byte)
              Byte.Count := Byte.Count + 1
              --}}}  
              --}}}  
        --}}}  
      Transfer.Mode > 1
        --{{{  Non DMA 1 byte at a time with a delay of between 2 and 32767 uSeconds
        WHILE Byte.Count < Packet.Size
          SEQ
            --{{{  Transfer a byte
            Command := Transfer.Information  -- Non DMA
            --{{{  Wait for interrupt
            IF
              (Execution.Status = SCSI.E.Good)
                SEQ
                  Accept.Interrupt ( WAIT.FOR.INTERRUPT,
                                     Interrupt.Time.Out,
                                     NCR.Status,
                                     Execution.Status)
                  --{{{  Interrogate
                  --{{{  Check for SCSI Bus Reset
                  IF
                    ((NCR.Status[Interrupt.Pointer] /\ SCSI.Reset.Detected.Mask) =
                    SCSI.Reset.Detected.Mask)
                      IF
                        Execution.Status = SCSI.E.Good
                          Execution.Status := SCSI.E.Bus.Reset
                        TRUE
                          SKIP
                    TRUE
                      --{{{  Interrogate registers
                      IF
                        ((NCR.Status[Status.Pointer] /\ Parity.Error.Mask) = Parity.Error.Mask)
                          IF
                            Execution.Status = SCSI.E.Good
                              Execution.Status := SCSI.E.SCSI.Bus.Parity.Error
                            TRUE
                              SKIP
                        TRUE
                          IF
                            Execution.Status = SCSI.E.Good
                              --{{{  Interrogate further
                              SEQ
                                SCSI.Phase :=  (NCR.Status[Status.Pointer] /\ SCSI.Bus.Phase.Mask)
                                IF
                                  ((NCR.Status[Interrupt.Pointer] /\ Disconnect.Mask) = Disconnect.Mask)
                                    --{{{  Disconnect Handler
                                    BOOL Finished:
                                    INT Next.SCSI.Phase:
                                    BYTE Msg.Length:
                                    [1]BYTE Message:
                                    SEQ
                                      Finished := FALSE
                                      Execution.Status := SCSI.E.Disconnected
                                      WHILE NOT Finished
                                        SEQ
                                          --{{{  Disconnect Handler
                                          SCSI.I.Disconnect.Handler( WAIT.FOR.INTERRUPT,
                                                                     Interrupt.Time.Out,
                                                                     SCSI.Phase,
                                                                     NCR.Status,
                                                                     Execution.Status)
                                          --}}}  
                                          --{{{  Execute phase requested by target
                                          CASE INT Next.SCSI.Phase
                                            Status.Phase
                                              SEQ
                                                Execution.Status := SCSI.E.Unexpected.Phase.Change
                                            Data.In.Phase
                                              SEQ
                                                Finished := TRUE
                                            Data.Out.Phase
                                              SEQ
                                                Execution.Status := SCSI.E.Unexpected.Phase.Change
                                            Command.Phase
                                              SEQ
                                                Execution.Status := SCSI.E.Unexpected.Phase.Change
                                            Message.Out.Phase
                                              SEQ
                                                --{{{  Message Out Phase
                                                SCSI.I.Message.Out.Phase(WAIT.FOR.INTERRUPT,
                                                                         Interrupt.Time.Out,
                                                                         Msg.Length,
                                                                         Message,
                                                                         Next.SCSI.Phase,
                                                                         NCR.Status,
                                                                         Finished,
                                                                         Execution.Status)
                                                Finished := TRUE
                                                --}}}  
                                            Message.In.Phase
                                              SEQ
                                                --{{{  Message IN Phase
                                                SCSI.I.Message.In.Phase ( WAIT.FOR.INTERRUPT,
                                                                          Interrupt.Time.Out,
                                                                          Next.SCSI.Phase,
                                                                          Msg.Length,
                                                                          Message,
                                                                          NCR.Status,
                                                                          Execution.Status)
                                                --}}}  
                                            ELSE
                                              --{{{  Software Error
                                              IF
                                                Execution.Status = SCSI.E.Good
                                                  SEQ
                                                    Execution.Status := SCSI.E.Software.Error
                                                TRUE
                                                  SKIP
                                              --}}}  
                                          --}}}  
                                    --}}}  
                                  TRUE
                                    IF
                                      (Operating.Mode = Initiator)
                                        IF
                                          ((NCR.Status[Interrupt.Pointer] /\ Bus.Service.Mask) <> Bus.Service.Mask)
                                            SEQ
                                              Execution.Status := SCSI.E.Data.In.Failure
                                          TRUE
                                            SKIP
                                      (Operating.Mode = Target)
                                        SKIP
                                      TRUE
                                        Execution.Status := SCSI.E.Software.Error
                              --}}}  
                            TRUE
                              SKIP
                      --}}}  
                  --}}}  
                  --}}}  
              TRUE
                SKIP
            --}}}  
            --{{{  Access FIFO
            Rx.Bytes[Byte.Count] := BYTE (FIFO /\ Low.Byte)
            Byte.Count := Byte.Count + 1
            --}}}  
            --{{{  Delay for n micro seconds
            PRI PAR
              --{{{  delay
              INT Now:
              TIMER Clock:
              SEQ
                Clock ? Now
                Clock ? AFTER (Now PLUS Transfer.Mode)
              --}}}  
              SKIP
            --}}}  
            --}}}  
        --}}}  
      TRUE
        Execution.Status := SCSI.E.Software.Error
    --}}}  
  --}}}  
:
--}}}  
