--{{{  Header
--#COMMENT "Copyright INMOS Limited 1990, 1991, 1992"
--#COMMENT "IMS F002C Product release SCSI software libraries"
--------------------------------------------------------------------------------
--
-- Date        : 19-Jan-1990 09:46:20
--
-- File Name   : TXESPLP.INC
--
-- Project     : F002 - SCSI Tram Software
--
-- Version     : 1.0
--
-- Division    : IQ Systems
--
-- Limitations : This software may be distributed in source form.
--
-- Description : An ESP tx transfer loop. ie one iteration of up to 64K
--               TX transfer to 53c94.
--
--------------------------------------------------------------------------------
-- Schedule of Amendments
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
--
--}}}  
--{{{  implementation
PROC ESP.LOOP ( VAL INT32 ESP.Transfer.Count,
                VAL INT   Operating.Mode,
                VAL INT   Data.Transfer.Mode,
                VAL INT   Buffer.Start,
                VAL INT   Buffer.Size,
                VAL INT16 IO.Pno,
                VAL INT16 Max.Packet.Size,
                VAL BYTE  Link.Operation,
                VAL INT   Command.Link,
                VAL INT   Data.Link,
                INT32     Data.Pointer,
                INT       SCSI.Phase,
                [3]INT    NCR.Status,
                INT16     Execution.Status)

  --{{{  Uses
  #USE "b422irpt.lib"
  #USE "dmaloop.lib"
  --}}}  
  --{{{  Declarations
  INT   Packet.Count:
  INT   Packets:
  INT   Remaining.Bytes:
  INT   Rem:
  INT   Link.Buffer.Pointer:
  INT   Tx.DMA.Buffer.Pointer:
  INT16 Returned.IO.Pno.1, Returned.IO.Pno.2:
  INT16 Returned.Packet.Size.16.1, Returned.Packet.Size.16.2:
  INT   Packet.Size, Half.Packet.Size:
  BOOL  Remainder, Large :
  VAL   []BYTE Data.Count.Bytes RETYPES ESP.Transfer.Count:
  --}}}  
  --{{{  Implementation
  SEQ
    --{{{  DEBUG
    --}}}  
    --{{{  Initialise
    --{{{  Odd's and Sod's
    Large := FALSE
    
    Remainder := FALSE
    
    Command := Flush.FIFO
    
    Packet.Size := INT Max.Packet.Size
    
    Half.Packet.Size := Packet.Size / 2
    --}}}  
    --{{{  Prepare NCR53c94 for DMA
    
    Transfer.Count.MSB := INT Data.Count.Bytes[1]
    Transfer.Count.LSB := INT Data.Count.Bytes[0]
    
    Command := DMA.Nop -- This weird bit loads the transfer count
    
    IF
      (Operating.Mode = Initiator)
        IF
          (Data.Transfer.Mode > (-1))
            Command := Transfer.Information.DMA
          TRUE
            SKIP
      (Operating.Mode = Target)
        IF
          (Data.Transfer.Mode > (-1))
            Command := Send.Data.DMA
          TRUE
            SKIP
      TRUE
        Execution.Status := SCSI.E.Software.Error
    --}}}  
    --{{{  Calculate no of packets and remaining bytes
    Packet.Count    := INT (ESP.Transfer.Count / (INT32 Max.Packet.Size))
    
    Packets := Packet.Count
    
    Remaining.Bytes := INT (ESP.Transfer.Count \ (INT32 Max.Packet.Size))
    
    Rem := Remaining.Bytes
    --}}}  
    --{{{  Initialise Buffer pointers
    Link.Buffer.Pointer := Buffer.Start
    Tx.DMA.Buffer.Pointer := Buffer.Start
    --}}}  
    --{{{  Prepare Link I/O
    --{{{  Synchronise buffers
    CASE Link.Operation
      Single.Link
        --{{{  Synchronise 1 link engine
        TO.LINK.IN.DRIVER[Command.Link] ! Synchronise.Tx.Buffers
        --}}}  
      ELSE
        --{{{  Synchronise 2 link engines
        PAR
          TO.LINK.IN.DRIVER[Command.Link]  ! Synchronise.Tx.Buffers
          TO.LINK.IN.DRIVER[Data.Link]     ! Synchronise.Tx.Buffers
        --}}}  
    --}}}  
    --{{{  Request Packets from host
    IF
      Packet.Count > 0
        SEQ
          --{{{  Kick off link engines
          CASE Link.Operation
            Single.Link
              --{{{  Kick off only one link engine
              SEQ
                TO.LINK.IN.DRIVER[Command.Link]  ! Tx.Buffer.Select; IO.Pno; Link.Buffer.Pointer; INT16 Packet.Size
                TO.LINK.OUT.DRIVER[Command.Link] ! Tx.Packet.Request; IO.Pno; Data.Pointer; INT16 Packet.Size
              --}}}  
            ELSE
              --{{{  Kick off 2 link engines
              --{{{  Abbreviations
              VAL INT32 DP.Plus.Half.Packet  IS Data.Pointer + (INT32 Half.Packet.Size):
              VAL INT   LBP.Plus.Half.Packet IS Link.Buffer.Pointer + Half.Packet.Size:
              --}}}  
              PAR
                --{{{  Command Link
                SEQ
                  TO.LINK.IN.DRIVER[Command.Link]  ! Tx.Buffer.Select;
                                                     IO.Pno;
                                                     Link.Buffer.Pointer;
                                                     INT16 Half.Packet.Size
                  TO.LINK.OUT.DRIVER[Command.Link] ! Tx.Packet.Request;
                                                     IO.Pno;
                                                     Data.Pointer;
                                                     INT16 Half.Packet.Size
                --}}}  
                --{{{  Data Link
                SEQ
                  TO.LINK.IN.DRIVER[Data.Link]  ! Tx.Buffer.Select;
                                                  IO.Pno;
                                                  LBP.Plus.Half.Packet;
                                                  INT16 Half.Packet.Size
                  TO.LINK.OUT.DRIVER[Data.Link] ! Tx.Packet.Request;
                                                  IO.Pno;
                                                  DP.Plus.Half.Packet;
                                                  INT16 Half.Packet.Size
                --}}}  
              --}}}  
          --}}}  
          Data.Pointer := Data.Pointer + (INT32 Packet.Size)
      TRUE
        SEQ
          --{{{  Kick off link engines
          CASE Link.Operation
            Single.Link
              --{{{  Kick off only one link engine
              SEQ
                TO.LINK.IN.DRIVER[Command.Link]  ! Tx.Buffer.Select; IO.Pno; Link.Buffer.Pointer; INT16 Remaining.Bytes
                TO.LINK.OUT.DRIVER[Command.Link] ! Tx.Packet.Request; IO.Pno; Data.Pointer; INT16 Remaining.Bytes
              --}}}  
            ELSE
              --{{{  Kick off 2 link engines
              --{{{  Abreviations
              VAL INT   Half.RB IS Remaining.Bytes / 2:
              VAL INT   Remainder IS Remaining.Bytes REM 2:
              VAL INT32 DP.Plus.Half.RB  IS Data.Pointer + (INT32 Half.RB):
              VAL INT   RBP.Plus.Half.RB IS Tx.DMA.Buffer.Pointer + Half.RB:
              --}}}  
              PAR
                --{{{  Command Link
                SEQ
                  TO.LINK.IN.DRIVER[Command.Link]  ! Tx.Buffer.Select;
                                                     IO.Pno;
                                                     Link.Buffer.Pointer;
                                                     INT16 Half.RB
                  TO.LINK.OUT.DRIVER[Command.Link] ! Tx.Packet.Request;
                                                     IO.Pno;
                                                     Data.Pointer;
                                                     INT16 Half.RB
                --}}}  
                --{{{  Data Link
                SEQ
                  TO.LINK.IN.DRIVER[Data.Link]  ! Tx.Buffer.Select;
                                                  IO.Pno;
                                                  RBP.Plus.Half.RB;
                                                  INT16 (Half.RB + Remainder)
                  TO.LINK.OUT.DRIVER[Data.Link] ! Tx.Packet.Request;
                                                  IO.Pno;
                                                  DP.Plus.Half.RB;
                                                  INT16 (Half.RB + Remainder)
                --}}}  
              --}}}  
          --}}}  
          Data.Pointer := Data.Pointer + (INT32 Remaining.Bytes)
    --}}}  
    --}}}  
    --}}}  
    --{{{  Transfer Data using appropriate data transfer mode
    IF
      Data.Transfer.Mode < 0
        --{{{  Transfer to bus using Non DMA Mode
        SEQ
          --{{{  Transfer using largest possible packets
          WHILE (( Packet.Count > 0)   AND
                  ( Execution.Status = SCSI.E.Good))
            SEQ
              --{{{  Wait for next packet
              Large := TRUE
              
              CASE Link.Operation
                Single.Link
                  SEQ
                    FROM.LINK.IN.DRIVER[Command.Link] ? Returned.IO.Pno.1; Returned.Packet.Size.16.1
                    --{{{  Check for errors
                    IF
                      (Returned.Packet.Size.16.1 <> (INT16 Packet.Size))
                        Execution.Status := SCSI.E.Bad.Data.Out.Returned.Packet.Size
                      TRUE
                        SKIP
                    --}}}  
                ELSE
                  PAR
                    FROM.LINK.IN.DRIVER[Command.Link] ? Returned.IO.Pno.1;
                                                        Returned.Packet.Size.16.2
                    FROM.LINK.IN.DRIVER[Data.Link] ? Returned.IO.Pno.1;
                                                     Returned.Packet.Size.16.2
              
              Packet.Count := Packet.Count - 1
              
              --}}}  
              --{{{  Switch Buffers
              Switch.Buffers(Buffer.Start,
                             Buffer.Size,
                             INT Max.Packet.Size,
                             Link.Buffer.Pointer,
                             Tx.DMA.Buffer.Pointer)
              --}}}  
              --{{{  Request Packets from host
              IF
                Packet.Count > 0
                  SEQ
                    --{{{  Kick off link engines
                    CASE Link.Operation
                      Single.Link
                        --{{{  Kick off only one link engine
                        SEQ
                          TO.LINK.IN.DRIVER[Command.Link]  ! Tx.Buffer.Select; IO.Pno; Link.Buffer.Pointer; INT16 Packet.Size
                          TO.LINK.OUT.DRIVER[Command.Link] ! Tx.Packet.Request; IO.Pno; Data.Pointer; INT16 Packet.Size
                        --}}}  
                      ELSE
                        --{{{  Kick off 2 link engines
                        --{{{  Abbreviations
                        VAL INT32 DP.Plus.Half.Packet  IS Data.Pointer + (INT32 Half.Packet.Size):
                        VAL INT   LBP.Plus.Half.Packet IS Link.Buffer.Pointer + Half.Packet.Size:
                        --}}}  
                        PAR
                          --{{{  Command Link
                          SEQ
                            TO.LINK.IN.DRIVER[Command.Link]  ! Tx.Buffer.Select;
                                                               IO.Pno;
                                                               Link.Buffer.Pointer;
                                                               INT16 Half.Packet.Size
                            TO.LINK.OUT.DRIVER[Command.Link] ! Tx.Packet.Request;
                                                               IO.Pno;
                                                               Data.Pointer;
                                                               INT16 Half.Packet.Size
                          --}}}  
                          --{{{  Data Link
                          SEQ
                            TO.LINK.IN.DRIVER[Data.Link]  ! Tx.Buffer.Select;
                                                            IO.Pno;
                                                            LBP.Plus.Half.Packet;
                                                            INT16 Half.Packet.Size
                            TO.LINK.OUT.DRIVER[Data.Link] ! Tx.Packet.Request;
                                                            IO.Pno;
                                                            DP.Plus.Half.Packet;
                                                            INT16 Half.Packet.Size
                          --}}}  
                        --}}}  
                    --}}}  
                    Data.Pointer := Data.Pointer + (INT32 Packet.Size)
                TRUE
                  SEQ
                    IF
                      (Remaining.Bytes > 0)
                        SEQ
                          --{{{  Kick off link engines
                          CASE Link.Operation
                            Single.Link
                              --{{{  Kick off only one link engine
                              SEQ
                                TO.LINK.IN.DRIVER[Command.Link]  ! Tx.Buffer.Select; IO.Pno; Link.Buffer.Pointer; INT16 Remaining.Bytes
                                TO.LINK.OUT.DRIVER[Command.Link] ! Tx.Packet.Request; IO.Pno; Data.Pointer; INT16 Remaining.Bytes
                              --}}}  
                            ELSE
                              --{{{  Kick off 2 link engines
                              --{{{  Abreviations
                              VAL INT   Half.RB IS Remaining.Bytes / 2:
                              VAL INT   Remainder IS Remaining.Bytes REM 2:
                              VAL INT32 DP.Plus.Half.RB  IS Data.Pointer + (INT32 Half.RB):
                              VAL INT   RBP.Plus.Half.RB IS Tx.DMA.Buffer.Pointer + Half.RB:
                              --}}}  
                              PAR
                                --{{{  Command Link
                                SEQ
                                  TO.LINK.IN.DRIVER[Command.Link]  ! Tx.Buffer.Select;
                                                                     IO.Pno;
                                                                     Link.Buffer.Pointer;
                                                                     INT16 Half.RB
                                  TO.LINK.OUT.DRIVER[Command.Link] ! Tx.Packet.Request;
                                                                     IO.Pno;
                                                                     Data.Pointer;
                                                                     INT16 Half.RB
                                --}}}  
                                --{{{  Data Link
                                SEQ
                                  TO.LINK.IN.DRIVER[Data.Link]  ! Tx.Buffer.Select;
                                                                  IO.Pno;
                                                                  RBP.Plus.Half.RB;
                                                                  INT16 (Half.RB + Remainder)
                                  TO.LINK.OUT.DRIVER[Data.Link] ! Tx.Packet.Request;
                                                                  IO.Pno;
                                                                  DP.Plus.Half.RB;
                                                                  INT16 (Half.RB + Remainder)
                                --}}}  
                              --}}}  
                          --}}}  
                      TRUE
                        SKIP
                    Data.Pointer := Data.Pointer + (INT32 Remaining.Bytes)
              --}}}  
              --{{{  Perform DMA
              Tx.Non.DMA.Loop ( Interrupt.Time.Out,
                                Packet.Size,
                                Data.Transfer.Mode,
                                Tx.DMA.Buffer.Pointer,
                                NCR.Status,
                                Execution.Status)
              --}}}  
          --{{{  If an error occurred, we must still input the packet arrival message
          IF
            ((Execution.Status <> SCSI.E.Good) AND (Packet.Count > 0))
              --{{{  Wait for next packet although we will not be able to use it.
              SEQ
                CASE Link.Operation
                  Single.Link
                    SEQ
                      FROM.LINK.IN.DRIVER[Command.Link] ? Returned.IO.Pno.1; Returned.Packet.Size.16.1
                      --{{{  Check for errors
                      IF
                        (Returned.Packet.Size.16.1 <> (INT16 Packet.Size))
                          Execution.Status := SCSI.E.Bad.Data.Out.Returned.Packet.Size
                        TRUE
                          SKIP
                      --}}}  
                  ELSE
                    PAR
                      FROM.LINK.IN.DRIVER[Command.Link] ? Returned.IO.Pno.1;
                                                          Returned.Packet.Size.16.2
                      FROM.LINK.IN.DRIVER[Data.Link] ? Returned.IO.Pno.1;
                                                       Returned.Packet.Size.16.2
              
                Packet.Count := Packet.Count - 1
                Remaining.Bytes := 0
              --}}}  
            TRUE
              SKIP
          --}}}  
          --}}}  
          --{{{  Transfer remaining bytes
          IF
            (Execution.Status = SCSI.E.Good)
              SEQ
                IF
                  ((Remaining.Bytes > 0)   AND
                   (Remaining.Bytes <> 0))
                    SEQ
                      --{{{  Wait for data
                      Remainder := TRUE
                      CASE Link.Operation
                        Single.Link
                          --{{{  Wait on a single link
                          SEQ
                            FROM.LINK.IN.DRIVER[Command.Link] ? Returned.IO.Pno.1; Returned.Packet.Size.16.1
                            --{{{  Check for errors
                            IF
                              (Returned.Packet.Size.16.1 <> (INT16 Remaining.Bytes))
                                Execution.Status := SCSI.E.Bad.Data.Out.Returned.Packet.Size
                              TRUE
                                SKIP
                            --}}}  
                          --}}}  
                        ELSE
                          --{{{  Wait on two links
                          PAR
                            FROM.LINK.IN.DRIVER[Command.Link] ? Returned.IO.Pno.1; Returned.Packet.Size.16.1
                            FROM.LINK.IN.DRIVER[Data.Link] ? Returned.IO.Pno.2; Returned.Packet.Size.16.2
                          --}}}  
                      --}}}  
                      --{{{  Switch Buffers
                      IF
                        Large
                          --{{{  Switch Buffers
                          Switch.Buffers(Buffer.Start,
                                         Buffer.Size,
                                         INT Max.Packet.Size,
                                         Link.Buffer.Pointer,
                                         Tx.DMA.Buffer.Pointer)
                          --}}}  
                        TRUE
                          SKIP
                      --}}}  
                      --{{{  Perform DMA
                      Tx.Non.DMA.Loop(Interrupt.Time.Out,
                                      Remaining.Bytes,
                                      Data.Transfer.Mode,
                                      Tx.DMA.Buffer.Pointer,
                                      NCR.Status,
                                      Execution.Status)
                      --}}}  
                  TRUE
                    SKIP
            TRUE
              SKIP
          --}}}  
          --{{{  Synchronise buffers
          CASE Link.Operation
            Single.Link
              --{{{  Synchronise 1 link engine
              TO.LINK.IN.DRIVER[Command.Link] ! Synchronise.Tx.Buffers
              --}}}  
            ELSE
              --{{{  Synchronise 2 link engines
              PAR
                TO.LINK.IN.DRIVER[Command.Link]  ! Synchronise.Tx.Buffers
                TO.LINK.IN.DRIVER[Data.Link]     ! Synchronise.Tx.Buffers
              --}}}  
          --}}}  
          --{{{  Check Status Info
          --{{{  Check for SCSI Bus Reset
          IF
            ((NCR.Status[Interrupt.Pointer] /\ SCSI.Reset.Detected.Mask) =
            SCSI.Reset.Detected.Mask)
              Execution.Status := SCSI.E.Bus.Reset
            TRUE
              SKIP
          --}}}  
          --{{{  Interrogate
          IF
            (Execution.Status = SCSI.E.Good)
              --{{{  Interrogate
              --{{{  Declarations
              [2]BYTE After.Transfer:
              --}}}  
              SEQ
                SCSI.Phase :=  (NCR.Status[Status.Pointer] /\ SCSI.Bus.Phase.Mask)
                After.Transfer[1] := BYTE (Transfer.Count.MSB /\ #FF)
                After.Transfer[0] := BYTE (Transfer.Count.LSB /\ #FF)
                VAL INT Transfer.Words RETYPES After.Transfer:
                SEQ
                  IF
                    ((NCR.Status[Status.Pointer] /\ Parity.Error.Mask) = Parity.Error.Mask)
                      Execution.Status := SCSI.E.SCSI.Bus.Parity.Error
                    TRUE
                      IF
                        Transfer.Words <> 0
                          Execution.Status :=  SCSI.E.Data.Out.Failure
                        TRUE
                          IF
                            ((FIFO.Flags /\ 7) <> 0)
                              Execution.Status := SCSI.E.Data.Out.Failure
                            TRUE
                              IF
                                ((NCR.Status[Interrupt.Pointer] /\ Disconnect.Mask) = Disconnect.Mask)
                                  Execution.Status := SCSI.E.Data.Out.Failure
                                TRUE
                                  IF
                                    (Operating.Mode = Initiator)
                                      IF
                                        ((NCR.Status[Interrupt.Pointer] /\ Bus.Service.Mask) <> Bus.Service.Mask)
                                          Execution.Status := SCSI.E.Data.Out.Failure
                                        TRUE
                                          SKIP
                                    (Operating.Mode = Target)
                                      SKIP
                                    TRUE
                                      Execution.Status := SCSI.E.Software.Error
              
                --Command := Flush.FIFO
              --}}}  
            TRUE
              SKIP
          --}}}  
          --}}}  
        --}}}  
      (Data.Transfer.Mode  >= (INT SCSI.Fast.DMA.Mode))
        --{{{  Transfer to bus using DMA Mode
        SEQ
          --{{{  Transfer using largest possible packets
          WHILE (( Packet.Count > 0) AND
                  ( Execution.Status = SCSI.E.Good))
            SEQ
              --{{{  Wait for next packet
              Large := TRUE
              
              CASE Link.Operation
                Single.Link
                  SEQ
                    FROM.LINK.IN.DRIVER[Command.Link] ? Returned.IO.Pno.1; Returned.Packet.Size.16.1
                    --{{{  Check for errors
                    IF
                      (Returned.Packet.Size.16.1 <> (INT16 Packet.Size))
                        Execution.Status := SCSI.E.Bad.Data.Out.Returned.Packet.Size
                      TRUE
                        SKIP
                    --}}}  
                ELSE
                  PAR
                    FROM.LINK.IN.DRIVER[Command.Link] ? Returned.IO.Pno.1;
                                                        Returned.Packet.Size.16.2
                    FROM.LINK.IN.DRIVER[Data.Link] ? Returned.IO.Pno.1;
                                                     Returned.Packet.Size.16.2
              
              Packet.Count := Packet.Count - 1
              
              --}}}  
              --{{{  Switch Buffers
              Switch.Buffers(Buffer.Start,
                             Buffer.Size,
                             INT Max.Packet.Size,
                             Link.Buffer.Pointer,
                             Tx.DMA.Buffer.Pointer)
              --}}}  
              --{{{  Request Packets from host
              IF
                Packet.Count > 0
                  SEQ
                    --{{{  Kick off link engines
                    CASE Link.Operation
                      Single.Link
                        --{{{  Kick off only one link engine
                        SEQ
                          TO.LINK.IN.DRIVER[Command.Link]  ! Tx.Buffer.Select; IO.Pno; Link.Buffer.Pointer; INT16 Packet.Size
                          TO.LINK.OUT.DRIVER[Command.Link] ! Tx.Packet.Request; IO.Pno; Data.Pointer; INT16 Packet.Size
                        --}}}  
                      ELSE
                        --{{{  Kick off 2 link engines
                        --{{{  Abbreviations
                        VAL INT32 DP.Plus.Half.Packet  IS Data.Pointer + (INT32 Half.Packet.Size):
                        VAL INT   LBP.Plus.Half.Packet IS Link.Buffer.Pointer + Half.Packet.Size:
                        --}}}  
                        PAR
                          --{{{  Command Link
                          SEQ
                            TO.LINK.IN.DRIVER[Command.Link]  ! Tx.Buffer.Select;
                                                               IO.Pno;
                                                               Link.Buffer.Pointer;
                                                               INT16 Half.Packet.Size
                            TO.LINK.OUT.DRIVER[Command.Link] ! Tx.Packet.Request;
                                                               IO.Pno;
                                                               Data.Pointer;
                                                               INT16 Half.Packet.Size
                          --}}}  
                          --{{{  Data Link
                          SEQ
                            TO.LINK.IN.DRIVER[Data.Link]  ! Tx.Buffer.Select;
                                                            IO.Pno;
                                                            LBP.Plus.Half.Packet;
                                                            INT16 Half.Packet.Size
                            TO.LINK.OUT.DRIVER[Data.Link] ! Tx.Packet.Request;
                                                            IO.Pno;
                                                            DP.Plus.Half.Packet;
                                                            INT16 Half.Packet.Size
                          --}}}  
                        --}}}  
                    --}}}  
                    Data.Pointer := Data.Pointer + (INT32 Packet.Size)
                TRUE
                  SEQ
                    IF
                      (Remaining.Bytes > 0)
                        SEQ
                          --{{{  Kick off link engines
                          CASE Link.Operation
                            Single.Link
                              --{{{  Kick off only one link engine
                              SEQ
                                TO.LINK.IN.DRIVER[Command.Link]  ! Tx.Buffer.Select; IO.Pno; Link.Buffer.Pointer; INT16 Remaining.Bytes
                                TO.LINK.OUT.DRIVER[Command.Link] ! Tx.Packet.Request; IO.Pno; Data.Pointer; INT16 Remaining.Bytes
                              --}}}  
                            ELSE
                              --{{{  Kick off 2 link engines
                              --{{{  Abreviations
                              VAL INT   Half.RB IS Remaining.Bytes / 2:
                              VAL INT   Remainder IS Remaining.Bytes REM 2:
                              VAL INT32 DP.Plus.Half.RB  IS Data.Pointer + (INT32 Half.RB):
                              VAL INT   RBP.Plus.Half.RB IS Tx.DMA.Buffer.Pointer + Half.RB:
                              --}}}  
                              PAR
                                --{{{  Command Link
                                SEQ
                                  TO.LINK.IN.DRIVER[Command.Link]  ! Tx.Buffer.Select;
                                                                     IO.Pno;
                                                                     Link.Buffer.Pointer;
                                                                     INT16 Half.RB
                                  TO.LINK.OUT.DRIVER[Command.Link] ! Tx.Packet.Request;
                                                                     IO.Pno;
                                                                     Data.Pointer;
                                                                     INT16 Half.RB
                                --}}}  
                                --{{{  Data Link
                                SEQ
                                  TO.LINK.IN.DRIVER[Data.Link]  ! Tx.Buffer.Select;
                                                                  IO.Pno;
                                                                  RBP.Plus.Half.RB;
                                                                  INT16 (Half.RB + Remainder)
                                  TO.LINK.OUT.DRIVER[Data.Link] ! Tx.Packet.Request;
                                                                  IO.Pno;
                                                                  DP.Plus.Half.RB;
                                                                  INT16 (Half.RB + Remainder)
                                --}}}  
                              --}}}  
                          --}}}  
                      TRUE
                        SKIP
                    Data.Pointer := Data.Pointer + (INT32 Remaining.Bytes)
              --}}}  
              --{{{  Perform DMA
              IF
                (Data.Transfer.Mode  = (INT SCSI.Fast.DMA.Mode))
                  --{{{  Fast DMA Mode
                  Fast.Tx.DMA.Loop ( Packet.Size,
                                     Tx.DMA.Buffer.Pointer,
                                     Tx.Buffer,
                                     Operating.Mode,
                                     B422.Status,
                                     Execution.Status)
                  --}}}  
                (Data.Transfer.Mode  >= (INT SCSI.Slow.DMA.Mode))
                  --{{{  Slow DMA
                  Slow.Tx.DMA.Loop ( Packet.Size,
                                     (Data.Transfer.Mode - (INT SCSI.Slow.DMA.Mode)),
                                     Tx.DMA.Buffer.Pointer,
                                     Execution.Status)
                  --}}}  
                TRUE
                  Execution.Status := SCSI.E.Invalid.Transfer.Mode
              --}}}  
          --{{{  If an error occurred, we must still input the packet arrival message
          IF
            ((Execution.Status <> SCSI.E.Good) AND (Packet.Count > 0))
              --{{{  Wait for next packet although we will not be able to use it.
              SEQ
                CASE Link.Operation
                  Single.Link
                    SEQ
                      FROM.LINK.IN.DRIVER[Command.Link] ? Returned.IO.Pno.1; Returned.Packet.Size.16.1
                      --{{{  Check for errors
                      IF
                        (Returned.Packet.Size.16.1 <> (INT16 Packet.Size))
                          Execution.Status := SCSI.E.Bad.Data.Out.Returned.Packet.Size
                        TRUE
                          SKIP
                      --}}}  
                  ELSE
                    PAR
                      FROM.LINK.IN.DRIVER[Command.Link] ? Returned.IO.Pno.1;
                                                          Returned.Packet.Size.16.2
                      FROM.LINK.IN.DRIVER[Data.Link] ? Returned.IO.Pno.1;
                                                       Returned.Packet.Size.16.2
              
                Packet.Count := Packet.Count - 1
                Remaining.Bytes := 0
              --}}}  
            TRUE
              SKIP
          --}}}  
          --}}}  
          --{{{  Transfer remaining bytes
          IF
            (Execution.Status = SCSI.E.Good)
              SEQ
                IF
                  ((Remaining.Bytes > 0)   AND
                   (Remaining.Bytes <> 0))
                    SEQ
                      --{{{  Wait for data
                      Remainder := TRUE
                      CASE Link.Operation
                        Single.Link
                          --{{{  Wait on a single link
                          SEQ
                            FROM.LINK.IN.DRIVER[Command.Link] ? Returned.IO.Pno.1; Returned.Packet.Size.16.1
                            --{{{  Check for errors
                            IF
                              (Returned.Packet.Size.16.1 <> (INT16 Remaining.Bytes))
                                Execution.Status := SCSI.E.Bad.Data.Out.Returned.Packet.Size
                              TRUE
                                SKIP
                            --}}}  
                          --}}}  
                        ELSE
                          --{{{  Wait on two links
                          PAR
                            FROM.LINK.IN.DRIVER[Command.Link] ? Returned.IO.Pno.1; Returned.Packet.Size.16.1
                            FROM.LINK.IN.DRIVER[Data.Link] ? Returned.IO.Pno.2; Returned.Packet.Size.16.2
                          --}}}  
                      --}}}  
                      --{{{  Switch Buffers
                      IF
                        Large
                          --{{{  Switch Buffers
                          Switch.Buffers(Buffer.Start,
                                         Buffer.Size,
                                         INT Max.Packet.Size,
                                         Link.Buffer.Pointer,
                                         Tx.DMA.Buffer.Pointer)
                          --}}}  
                        TRUE
                          SKIP
                      --}}}  
                      --{{{  Perform DMA
                      IF
                        (Data.Transfer.Mode  = (INT SCSI.Fast.DMA.Mode))
                          --{{{  Fast DMA Mode
                          Fast.Tx.DMA.Loop ( Remaining.Bytes,
                                             Tx.DMA.Buffer.Pointer,
                                             Tx.Buffer,
                                             Operating.Mode,
                                             B422.Status,
                                             Execution.Status)
                          --}}}  
                        (Data.Transfer.Mode  >= (INT SCSI.Slow.DMA.Mode))
                          --{{{  Slow DMA
                          Slow.Tx.DMA.Loop ( Remaining.Bytes,
                                             (Data.Transfer.Mode - (INT SCSI.Slow.DMA.Mode)),
                                             Tx.DMA.Buffer.Pointer,
                                             Execution.Status)
                          --}}}  
                        TRUE
                          Execution.Status := SCSI.E.Invalid.Transfer.Mode
                      --}}}  
                  TRUE
                    SKIP
            TRUE
              SKIP
          --}}}  
          --{{{  Synchronise buffers
          CASE Link.Operation
            Single.Link
              --{{{  Synchronise 1 link engine
              TO.LINK.IN.DRIVER[Command.Link] ! Synchronise.Tx.Buffers
              --}}}  
            ELSE
              --{{{  Synchronise 2 link engines
              PAR
                TO.LINK.IN.DRIVER[Command.Link]  ! Synchronise.Tx.Buffers
                TO.LINK.IN.DRIVER[Data.Link]     ! Synchronise.Tx.Buffers
              --}}}  
          --}}}  
          --{{{  Wait for interrupt
          IF
            ((Execution.Status = SCSI.E.Good) OR
             ((B422.Status /\ Interrupt.Occurred) = Interrupt.Occurred))
              SEQ
                Accept.Interrupt ( WAIT.FOR.INTERRUPT,
                                   Interrupt.Time.Out,
                                   NCR.Status,
                                   Execution.Status)
                --{{{  Check for SCSI Bus Reset
                IF
                  ((NCR.Status[Interrupt.Pointer] /\ SCSI.Reset.Detected.Mask) =
                  SCSI.Reset.Detected.Mask)
                    SEQ
                      --{{{  DEBUG
                      --}}}  
                      --{{{  Flag error
                      IF
                        Execution.Status = SCSI.E.Good
                          Execution.Status := SCSI.E.Bus.Reset
                        TRUE
                          SKIP
                      --}}}  
                  TRUE
                    SKIP
                --}}}  
                --{{{  Interrogate
                IF
                  (Execution.Status = SCSI.E.Good)
                    --{{{  Interrogate
                    --{{{  Declarations
                    [2]BYTE After.Transfer:
                    --}}}  
                    SEQ
                      SCSI.Phase :=  (NCR.Status[Status.Pointer] /\ SCSI.Bus.Phase.Mask)
                      After.Transfer[1] := BYTE (Transfer.Count.MSB /\ #FF)
                      After.Transfer[0] := BYTE (Transfer.Count.LSB /\ #FF)
                      VAL INT Transfer.Words RETYPES After.Transfer:
                      SEQ
                        IF
                          ((NCR.Status[Status.Pointer] /\ Parity.Error.Mask) = Parity.Error.Mask)
                            SEQ
                              --{{{  DEBUG
                              --}}}  
                              Execution.Status := SCSI.E.SCSI.Bus.Parity.Error
                          TRUE
                            IF
                              Transfer.Words <> 0
                                SEQ
                                  --{{{  DEBUG
                                  --}}}  
                                  Execution.Status :=  SCSI.E.Data.Out.Failure
                              TRUE
                                IF
                                  ((FIFO.Flags /\ 7) <> 0)
                                    SEQ
                                      --{{{  DEBUG
                                      --}}}  
                                      Execution.Status := SCSI.E.Data.Out.Failure
                                  TRUE
                                    IF
                                      ((NCR.Status[Interrupt.Pointer] /\ Disconnect.Mask) = Disconnect.Mask)
                                        SEQ
                                          --{{{  DEBUG
                                          --}}}  
                                          Execution.Status := SCSI.E.Data.Out.Failure
                                      TRUE
                                        IF
                                          (Operating.Mode = Initiator)
                                            IF
                                              ((NCR.Status[Interrupt.Pointer] /\ Bus.Service.Mask) <> Bus.Service.Mask)
                                                SEQ
                                                  --{{{  DEBUG
                                                  --}}}  
                                                  Execution.Status := SCSI.E.Data.Out.Failure
                                              TRUE
                                                SKIP
                                          (Operating.Mode = Target)
                                            SKIP
                                          TRUE
                                            Execution.Status := SCSI.E.Software.Error
                    
                      Command := Flush.FIFO
                    --}}}  
                  TRUE
                    SKIP
                --}}}  
            TRUE
              SKIP
          --}}}  
        --}}}  
      TRUE
        Execution.Status := SCSI.E.Invalid.Transfer.Mode
    --}}}  
    --{{{  DEBUG
    --}}}  
  --}}}  
:
--}}}  
